#
#  Copyright (c) 2015 - 2023, Intel Corporation
#  SPDX-License-Identifier: BSD-3-Clause
#

#TUTORIAL_BASE
#
#  Setup script that installs geopm into a temporary location, and
#  then builds the tutorials from that install based on the
#  geopm-tutorial.tar.gz file generated by make dist.
#

set -e
export SRC_BASE_DIR=$(dirname $(dirname $(dirname $(realpath $0))))
export LINK_DIR=$SRC_BASE_DIR/integration/test/test_tutorial_base

function get_config
{
    # Extract variable value out of the config.log
    grep "^$1=" $SRC_BASE_DIR/config.log | sed "s|$1='\(.*\)'|\1|"
}

# Determine the compiler
if [[ $(get_config CC) == 'icc' ]]; then
    compiler=intel
elif [[ $(get_config CC) == 'gcc' ]]; then
    compiler=gnu
else
    echo "Error: $0 unable to determine compiler" 1>&2 && false
fi

# Clean out last install if it exists
if [[ -L $LINK_DIR ]]; then
    rm -rf $(readlink $LINK_DIR) $LINK_DIR
fi

# Make directory for temporary install and tutorial
mkdir -p $HOME/.local/tmp/
temp_dir=$(mktemp -d $HOME/.local/tmp/geopm-tutorial-test.XXXXXXXX)

# Create a symbolic link to the temporary install directory
ln -s $temp_dir $LINK_DIR

# Install geopm into the temporary location
unset GEOPM_INSTALL GEOPM_LAUNCHER GEOPM_BIN GEOPM_INC GEOPM_LIB GEOPM_CFLAGS GEOPM_LDFLAGS
destdir=$temp_dir/build
prefix=$(get_config prefix)
cd $SRC_BASE_DIR/service
DESTDIR=$destdir make install
make dist
cd ..
DESTDIR=$destdir make install
make dist

# Build the tutorials
cd -P $LINK_DIR
tar xvf $SRC_BASE_DIR/geopm-tutorial.tar.gz
cd geopm-tutorial
export GEOPM_INSTALL=$destdir/$prefix
./tutorial_build_$compiler.sh

# Print success message
echo SUCCESS: $0
